'******************************************************************************
'*  Name    : ENC28J60_Driver.INC                                             *
'*  Author  : Gabi MIHAILA                                                    *
'*  Notice  : Public  Release                                                 *
'*          :                                                                 *
'******************************************************************************
'*  Date    : 04-Dec-06                                                       *
'*  Version : 0.0.2.1                                                        *
'*  Compiler: P.D.S - Proton v.3.2.5.2                                        *
'*  Hosts   : Microchip 18 series                                             *
'*  Notes   : - ENC28J60 - "Demo" version Driver                              *
'*          : - Basic SPI Functions and registers declares                    *
'*          ; - follows the ENC28J60 datasheet dated July 2006                *
'*          ;   DS39662B                                                      *
'******************************************************************************


;***
; un-comment if you want to test the chip Revision ID example
;HSERIAL_BAUD = 115200          		' Set baud rate to 9600
;HSERIAL_RCSTA = %10110000              ' Enable serial port and continuous receive
;HSERIAL_TXSTA = %00100100              ' Enable transmit and asynchronous mode 
;HSERIAL_CLEAR  = On		            ' Enable Error clearing on received characters
;***



;******************************************************************************
;* Banked Registers locations
;******************************************************************************
;* Bank 0 registers  
;
Symbol ADR_ERDPTL   = $00              ; address of the ERDPTL register
Symbol ADR_ERDPTH 	= $01              ; address of the ERDPTH register
Symbol ADR_EWRPTL	= $02              ;
Symbol ADR_EWRPTH	= $03
Symbol ADR_ETXSTL	= $04
Symbol ADR_ETXSTH	= $05
Symbol ADR_ETXNDL	= $06
Symbol ADR_ETXNDH	= $07
Symbol ADR_ERXSTL	= $08
Symbol ADR_ERXSTH	= $09
Symbol ADR_ERXNDL	= $0A
Symbol ADR_ERXNDH	= $0B
Symbol ADR_ERXRDPTL	= $0C
Symbol ADR_ERXRDPTH	= $0D
Symbol ADR_ERXWRPTL	= $0E
Symbol ADR_ERXWRPTH	= $0F
Symbol ADR_EDMASTL  = $10
Symbol ADR_EDMASTH	= $11
Symbol ADR_EDMANDL	= $12
Symbol ADR_EDMANDH	= $13
Symbol ADR_EDMADSTL	= $14
Symbol ADR_EDMADSTH	= $15
Symbol ADR_EDMACSL	= $16
Symbol ADR_EDMACSH	= $17
Symbol ADR_EIE		= $1B
Symbol ADR_EIR		= $1C
Symbol ADR_ESTAT	= $1D
Symbol ADR_ECON2	= $1E
Symbol ADR_ECON1	= $1F
;******************************************************************************
;* Bank 1 registers  
;
Symbol ADR_EHT0		= $100             ; $ 1= bank 1, then Register address , ie $00
Symbol ADR_EHT1		= $101
Symbol ADR_EHT2		= $102
Symbol ADR_EHT3		= $103
Symbol ADR_EHT4		= $104
Symbol ADR_EHT5		= $105
Symbol ADR_EHT6		= $106
Symbol ADR_EHT7		= $107
Symbol ADR_EPMM0	= $108
Symbol ADR_EPMM1	= $109
Symbol ADR_EPMM2	= $10A
Symbol ADR_EPMM3	= $10B
Symbol ADR_EPMM4	= $10C
Symbol ADR_EPMM5	= $10D
Symbol ADR_EPMM6	= $10E
Symbol ADR_EPMM7	= $10F
Symbol ADR_EPMCSL	= $110
Symbol ADR_EPMCSH	= $111
Symbol ADR_EPMOL	= $114
Symbol ADR_EPMOH	= $115
Symbol ADR_EWOLIE	= $116   ;?
Symbol ADR_EWOLIR	= $117   ;?
Symbol ADR_ERXFCON	= $118
Symbol ADR_EPKTCNT	= $119
;******************************************************************************
;* Bank 2 registers  
;
Symbol ADR_MACON1	= $200
Symbol ADR_MACON2	= $201   ;?
Symbol ADR_MACON3	= $202
Symbol ADR_MACON4	= $203
Symbol ADR_MABBIPG	= $204
Symbol ADR_MAIPGL	= $206
Symbol ADR_MAIPGH	= $207
Symbol ADR_MACLCON1	= $208
Symbol ADR_MACLCON2	= $209
Symbol ADR_MAMXFLL	= $20A
Symbol ADR_MAMXFLH	= $20B
Symbol ADR_MAPHSUP	= $20D   ;?
Symbol ADR_MICON	= $211   ;?
Symbol ADR_MICMD	= $212
Symbol ADR_MIREGADR	= $214
Symbol ADR_MIWRL	= $216
Symbol ADR_MIWRH	= $217
Symbol ADR_MIRDL	= $218
Symbol ADR_MIRDH	= $219
;******************************************************************************
;// Bank 3 registers  
;
;Symbol ADR_MAADR1	= $300
;Symbol ADR_MAADR0	= $301
;Symbol ADR_MAADR3	= $302
;Symbol ADR_MAADR2	= $303
;Symbol ADR_MAADR5	= $304
;Symbol ADR_MAADR4	= $305
;
Symbol ADR_MAADR5	= $300    ; as in new datasheet 07/2006
Symbol ADR_MAADR6	= $301    ; <-- this is the leftmost MAC address number
Symbol ADR_MAADR3	= $302
Symbol ADR_MAADR4	= $303
Symbol ADR_MAADR1	= $304    ; this is the rightmost MAC address
Symbol ADR_MAADR2	= $305
;
Symbol ADR_EBSTSD	= $306
Symbol ADR_EBSTCON	= $307
Symbol ADR_EBSTCSL	= $308
Symbol ADR_EBSTCSH	= $309
Symbol ADR_MISTAT	= $30A
Symbol ADR_EREVID	= $312
Symbol ADR_ECOCON	= $315
Symbol ADR_EFLOCON	= $317
Symbol ADR_EPAUSL	= $318
Symbol ADR_EPAUSH	= $319
;******************************************************************************
;* PHY Registers Locations  
;
Symbol ADR_PHCON1	= $00
Symbol ADR_PHSTAT1	= $01
Symbol ADR_PHID1	= $02
Symbol ADR_PHID2	= $03
Symbol ADR_PHCON2	= $10
Symbol ADR_PHSTAT2	= $11
Symbol ADR_PHIE	    = $12
Symbol ADR_PHIR	    = $13
Symbol ADR_PHLCON	= $14
;******************************************************************************
;* Register's Individual Bits
;******************************************************************************
;* ETH/MAC/MII bits
;
;------------------------------------------------------------------------------
;* EIE bits
;
;Dim EIE As Byte                                  ; ETHERNET INTERRUPT ENABLE REGISTER
;
;Symbol	EIE_INTIE      = EIE.7	                 ; Global INT Interrupt Enable
;Symbol	EIE_PKTIE      = EIE.6                   ; Receive Packet Pending Interrupt Enable bit
;Symbol	EIE_DMAIE	   = EIE.5                   ; DMA Interrupt Enable bit
;Symbol	EIE_LINKIE	   = EIE.4                   ; Link Status Change Interrupt Enable bit
;Symbol	EIE_TXIE	   = EIE.3                   ; Transmit Enable bit
;Symbol	EIE_WOLIE	   = EIE.2                   ; Global WOL Interrupt Enable bit
;Symbol	EIE_TXERIE	   = EIE.1                   ; Transmit Error Interrupt Enable bit
;Symbol	EIE_RXERIE	   = EIE.0                   ; Receive Error Interrupt Enable bit
;------------------------------------------------------------------------------
;* EIR bits  
;
Dim EIR As Byte                                  ; ETHERNET INTERRUPT REQUEST (FLAG) REGISTER
;                                                ;
Symbol	EIR_PKTIF	   = EIR.6                   ; Receive Packet Pending Interrupt Flag bit
Symbol	EIR_DMAIF	   = EIR.5                   ; DMA Interrupt Flag bit
Symbol	EIR_LINKIF	   = EIR.4                   ; Link Change Interrupt Flag bit
Symbol	EIR_TXIF	   = EIR.3                   ; Transmit Interrupt Flag bit
Symbol	EIR_WOLIF	   = EIR.2                   ; WOL Interrupt Flag bit
Symbol	EIR_TXERIF	   = EIR.1                   ; Transmit Error Interrupt Flag bit
Symbol	EIR_RXERIF	   = EIR.0                   ; Receive Error Interrupt Flag bit
;------------------------------------------------------------------------------
;* ESTAT bits  
;
Dim ESTAT As Byte                                ; ETHERNET STATUS REGISTER
;                              
Symbol	ESTAT_INT	   = ESTAT.7                 ; INT Interrupt Flag bit
Symbol	ESTAT_LATECOL  = ESTAT.4                 ; Late Collision Error bit
Symbol	ESTAT_RXBUSY   = ESTAT.2                 ; Receive Busy bit
Symbol	ESTAT_TXABRT   = ESTAT.1                 ; Transmit Abort Error bit
Symbol	ESTAT_CLKRDY   = ESTAT.0                 ; Clock Ready bit
;------------------------------------------------------------------------------
;* ECON2 bits  
;
Dim ECON2 As Byte                                ; ETHERNET CONTROL REGISTER 2
;
Symbol	ECON2_AUTOINC  = ECON2.7                 ; Automatic Buffer Pointer Increment Enable bit
Symbol	ECON2_PKTDEC   = ECON2.6                 ; Packet Decrement bit
Symbol	ECON2_PWRSV	   = ECON2.5                 ; Power Save Enable bit
Symbol	ECON2_VRPS	   = ECON2.3                 ; Voltage Regulator Power Save Enable bit
;------------------------------------------------------------------------------
;* ECON1 bits 
;
; The ECON1 register, shown in Register3-1, is used to
; control the main functions of the ENC28J60. Receive
; enable, transmit request, DMA control and bank select
; bits can all be found in ECON1. 
;
Dim ECON1 As Byte                                ; ETHERNET CONTROL REGISTER 1
;
Symbol	ECON1_TXRST	   = ECON1.7                 ; Transmit Logic Reset bit
Symbol	ECON1_RXRST	   = ECON1.6                 ; Receive Logic Reset bit
Symbol	ECON1_DMAST	   = ECON1.5                 ; DMA Start and Busy Status bit
Symbol	ECON1_CSUMEN   = ECON1.4                 ; DMA Checksum Enable bit
Symbol	ECON1_TXRTS	   = ECON1.3                 ; Transmit Request to Send bit
Symbol	ECON1_RXEN	   = ECON1.2                 ; Receive Enable bit
Symbol	ECON1_BSEL1	   = ECON1.1                 ; Bank Select bits 1
Symbol	ECON1_BSEL0	   = ECON1.0                 ; Bank Select bits 0
;------------------------------------------------------------------------------
;* EWOLIE bits  
;                                                ;* not anymore in new datasheet
;Dim EWOLIE As Byte                               ; 
;
;Symbol	EWOLIE_UCWOLIE = EWOLIE.7                ;
;Symbol	EWOLIE_AWOLIE  = EWOLIE.6                ;
;Symbol	EWOLIE_PMWOLIE = EWOLIE.4                ;
;Symbol	EWOLIE_MPWOLIE = EWOLIE.3                ;
;Symbol	EWOLIE_HTWOLIE = EWOLIE.2                ;
;Symbol	EWOLIE_MCWOLIE = EWOLIE.1                ;
;Symbol	EWOLIE_BCWOLIE = EWOLIE.0                ;
;------------------------------------------------------------------------------
;* EWOLIR bits  
;                                                ; * not anymore in new datasheet
;Dim EWOLIR As Byte
;
;Symbol	EWOLIR_UCWOLIF = EWOLIR.7                ;
;Symbol	EWOLIR_AWOLIF  = EWOLIR.6                ;
;Symbol	EWOLIR_PMWOLIF = EWOLIR.4                ;
;Symbol	EWOLIR_MPWOLIF = EWOLIR.3                ;
;Symbol	EWOLIR_HTWOLIF = EWOLIR.2                ;
;Symbol	EWOLIR_MCWOLIF = EWOLIR.1                ;
;Symbol	EWOLIR_BCWOLIF = EWOLIR.0                ;
;------------------------------------------------------------------------------
;* ERXFCON bits  
;
Dim ERXFCON As Byte                              ; ETHERNET RECEIVE FILTER CONTROL REGISTER
;
Symbol	ERXFCON_UCEN   = ERXFCON.7               ; Unicast Filter Enable bit
Symbol	ERXFCON_ANDOR  = ERXFCON.6               ; AND/OR Filter Select bit
Symbol	ERXFCON_CRCEN  = ERXFCON.5               ; Post-Filter CRC Check Enable bit
Symbol	ERXFCON_PMEN   = ERXFCON.4               ; Pattern Match Filter Enable bit
Symbol	ERXFCON_MPEN   = ERXFCON.3               ; Magic Packet Filter Enable bit
Symbol	ERXFCON_HTEN   = ERXFCON.2               ; Hash Table Filter Enable bit
Symbol	ERXFCON_MCEN   = ERXFCON.1               ; Multicast Filter Enable bit
Symbol	ERXFCON_BCEN   = ERXFCON.0               ; Broadcast Filter Enable bit
;------------------------------------------------------------------------------
;* MACON1 bits
;
Dim MACON1 As Byte                               ; MAC CONTROL REGISTER 1
;  
;symbol	MACON1_LOOPBK  = MACON1.4                ; not anymore in new datasheet
Symbol	MACON1_TXPAUS  = MACON1.3                ; Pause Control Frame Transmission Enable bit
Symbol	MACON1_RXPAUS  = MACON1.2                ; Pause Control Frame Reception Enable bit
Symbol	MACON1_PASSALL = MACON1.1                ; Pass All Received Frames Enable bit
Symbol	MACON1_MARXEN  = MACON1.0                ; MAC Receive Enable bit
;------------------------------------------------------------------------------
;* MACON2 bits                     ;* not anymore in the new datasheet
;
;dim MACON2 as byte                               ;
;
;symbol	MACON2_MARST   = MACON2.7                ;
;symbol	MACON2_RNDRST  = MACON2.6                ;
;symbol	MACON2_MARXRST = MACON2.3                ;
;symbol	MACON2_RFUNRST = MACON2.2                ;
;symbol	MACON2_MATXRST = MACON2.1                ;
;symbol	MACON2_TFUNRST = MACON2.0                ;
;------------------------------------------------------------------------------
;* MACON3 bits  
;
Dim MACON3 As Byte                               ; MAC CONTROL REGISTER 3
;
Symbol	MACON3_PADCFG2 = MACON3.7                ; Automatic Pad and CRC Configuration bits
Symbol	MACON3_PADCFG1 = MACON3.6                ;
Symbol	MACON3_PADCFG0 = MACON3.5                ;
Symbol	MACON3_TXCRCEN = MACON3.4                ; Transmit CRC Enable bit
Symbol	MACON3_PHDRLEN = MACON3.3                ; Proprietary Header Enable bit
Symbol	MACON3_HFRMEN  = MACON3.2                ; Huge Frame Enable bit
Symbol	MACON3_FRMLNEN = MACON3.1                ; Frame Length Checking Enable bit
Symbol	MACON3_FULDPX  = MACON3.0                ; MAC Full-Duplex Enable bit
;------------------------------------------------------------------------------
;* MACON4 bits
;
Dim MACON4 As Byte                               ; MAC CONTROL REGISTER 4
; 
Symbol	MACON4_DEFER   = MACON4.6                ; Defer Transmission Enable bit (applies to half duplex only)
Symbol	MACON4_BPEN	   = MACON4.5                ; No Backoff During Backpressure Enable bit (applies to half duplex only)
Symbol	MACON4_NOBKOFF = MACON4.4                ; No Backoff Enable bit (applies to half duplex only)
Symbol	MACON4_LONGPRE = MACON4.1                ; maintain to zero (reserved) .. not anymore in new datasheet
Symbol	MACON4_PUREPRE = MACON4.0                ; maintain to zero (reserved) .. not anymore in new datasheet
;------------------------------------------------------------------------------
;* MAPHSUP bits
;
;dim MAPHSUP as byte                 ;* not anymore in new datasheet
;
;symbol	MAPHSUP_RSTRMII= MAPHSUP.3
;------------------------------------------------------------------------------
;* MICON bits 
;
;dim MICON as byte                   ;* not anymore in new datasheet
;
;symbol	MICON_RSTMII   = MICOM.7
;------------------------------------------------------------------------------
;* MICMD bits  
;
Dim MICMD As Byte                                ; MII COMMAND REGISTER
;
Symbol	MICMD_MIISCAN  = MICMD.1                 ; MII Scan Enable bit
Symbol	MICMD_MIIRD	   = MICMD.0                 ; MII Read Enable bit
;------------------------------------------------------------------------------
;* EBSTCON bits  
;
;Dim EBSTCON As Byte                              ; ETHERNET SELF-TEST CONTROL REGISTER
;
;Symbol	EBSTCON_PSV2   = EBSTCON.7               ; Pattern Shift Value bits
;Symbol	EBSTCON_PSV1   = EBSTCON.6               ;
;Symbol	EBSTCON_PSV0   = EBSTCON.5               ;
;Symbol	EBSTCON_PSEL   = EBSTCON.4               ; Port Select bit
;Symbol	EBSTCON_TMSEL1 = EBSTCON.3               ; Test Mode Select bits
;Symbol	EBSTCON_TMSEL0 = EBSTCON.2               ;
;Symbol	EBSTCON_TME	   = EBSTCON.1               ; Test Mode Enable bit
;Symbol	EBSTCON_BISTST = EBSTCON.0               ; Built-in Self-Test Start/Busy bit
;------------------------------------------------------------------------------
;* MISTAT bits  
;
Dim MISTAT As Byte                               ; MII STATUS REGISTER
;
Symbol	MISTAT_NVALID  = MISTAT.2                ; MII Management Read Data Not Valid bit
Symbol	MISTAT_SCAN	   = MISTAT.1                ; MII Management Scan Operation bit
Symbol	MISTAT_BUSY	   = MISTAT.0                ; MII Management Busy bit
;------------------------------------------------------------------------------

;* Revision ID
Dim EREVID As Byte
;
;* ECOCON bits  
;
;Dim ECOCON As Byte                               ; CLOCK OUTPUT CONTROL REGISTER
;
;Symbol	ECOCON_COCON2  = ECOCON.2                ; Clock Output Configuration bits
;Symbol	ECOCON_COCON1  = ECOCON.1                ;
;Symbol	ECOCON_COCON0  = ECOCON.0                ;
;------------------------------------------------------------------------------
;* EFLOCON bits  
;
;Dim EFLOCON As Byte                              ; ETHERNET FLOW CONTROL REGISTER
;
;Symbol	EFLOCON_FULDPXS= EFLOCON.2               ; Read-Only MAC Full-Duplex Shadow bit
;Symbol	EFLOCON_FCEN1  = EFLOCON.1               ; Flow Control Enable bits
;Symbol	EFLOCON_FCEN0  = EFLOCON.0               ;
;------------------------------------------------------------------------------
;
;******************************************************************************
;* PHY registers bits
;******************************************************************************
; all PHY regs are word sized
;------------------------------------------------------------------------------
;* PHCON1 bits  
;
Dim PHCON1 As Word                               ; PHY CONTROL REGISTER 1
;
Symbol	PHCON1_PRST	   = PHCON1H.7	             ; PHY Software Reset bit
Symbol	PHCON1_PLOOPBK = PHCON1H.6	             ; PHY Loopback bit
Symbol	PHCON1_PPWRSV  = PHCON1H.3	             ; PHY Power-Down bit
Symbol	PHCON1_PDPXMD  = PHCON1H.0	             ; PHY Duplex Mode bit
;------------------------------------------------------------------------------
;* PHSTAT1 bits  
;
Dim PHSTAT1 As Word                              ; PHYSICAL LAYER STATUS REGISTER 1
;
Symbol	PHSTAT1_PFDPX  = PHSTAT1H.4	             ; PHY Full-Duplex Capable bit
Symbol	PHSTAT1_PHDPX  = PHSTAT1H.3	             ; PHY Half-Duplex Capable bit
Symbol	PHSTAT1_LLSTAT = PHSTAT1.2	             ; PHY Latching Link Status bit
Symbol	PHSTAT1_JBSTAT = PHSTAT1.1	             ; PHY Latching Jabber Status bit
;------------------------------------------------------------------------------
;* PHID1 bits
;
;Dim PHID1 As Word                                ; ETHERNET COntroller IDENTIFIERS regs
;
;Symbol  PHID1_OUI3     = PHID1H.7                ; The PHY Identifier, as part of Microchips
;Symbol  PHID1_OUI4     = PHID1H.6                ; corporate Organizationally Unique Identifier (OUI)
;Symbol  PHID1_OUI5     = PHID1H.5                ; (OUI3:OUI24)
;Symbol  PHID1_OUI6     = PHID1H.4                ; spanned as (OUI3:OUI18) in PHID1 , and **
;Symbol  PHID1_OUI7     = PHID1H.3                ;
;Symbol  PHID1_OUI8     = PHID1H.2                ; OUI1 and OUI2 always equal zero
;Symbol  PHID1_OUI9     = PHID1H.1                ;
;Symbol  PHID1_OUI10    = PHID1H.0                ;
;Symbol  PHID1_OUI11    = PHID1.7                 ;
;Symbol  PHID1_OUI12    = PHID1.6                 ;
;Symbol  PHID1_OUI13    = PHID1.5                 ;
;Symbol  PHID1_OUI14    = PHID1.4                 ;
;Symbol  PHID1_OUI15    = PHID1.3                 ;
;Symbol  PHID1_OUI16    = PHID1.2                 ;
;Symbol  PHID1_OUI17    = PHID1.1                 ;
;Symbol  PHID1_OUI18    = PHID1.0                 ;
;------------------------------------------------------------------------------
;// PHID2 bits  
;
;Dim PHID2 As Word                                ; ETHERNET COntroller IDENTIFIERS regs
;
;Symbol	PHID2_OUI24	   = PHID2H.7                ; ** and (OUI19:OUI24) in PHID2
;Symbol	PHID2_OUI23    = PHID2H.6                ;
;Symbol	PHID2_OUI22	   = PHID2H.5                ;
;Symbol	PHID2_OUI21	   = PHID2H.4                ;
;Symbol	PHID2_OUI20	   = PHID2H.3                ;
;Symbol	PHID2_OUI19	   = PHID2H.2                ;
;Symbol	PHID2_PPN5	   = PHID2H.1                ; The part number of the PHY module bits
;Symbol	PHID2_PPN4	   = PHID2H.0                ;
;Symbol	PHID2_PPN3	   = PHID2.7                 ;
;Symbol	PHID2_PPN2	   = PHID2.6                 ;
;Symbol	PHID2_PPN1	   = PHID2.5                 ;
;Symbol	PHID2_PPN0	   = PHID2.4                 ;
;Symbol	PHID2_PREV3	   = PHID2.3                 ; The revision level of the PHY module bits
;Symbol	PHID2_PREV2	   = PHID2.2                 ;
;Symbol	PHID2_PREV1	   = PHID2.1                 ;
;Symbol	PHID2_PREV0	   = PHID2.0                 ;
;------------------------------------------------------------------------------
;* PHCON2 bits  
;
Dim PHCON2 As Word                               ; PHY CONTROL REGISTER 2
;
Symbol	PHCON2_FRCLNK  = PHCON2H.6               ; PHY Force Linkup bit
Symbol	PHCON2_TXDIS   = PHCON2H.5               ; Twisted-Pair Transmitter Disable bit
Symbol	PHCON2_JABBER  = PHCON2H.2               ; Jabber Correction Disable bit
Symbol	PHCON2_HDLDIS  = PHCON2H.0               ; PHY Half-Duplex Loopback Disable bit
;------------------------------------------------------------------------------
;* PHSTAT2 bits  
;
Dim PHSTAT2 As Word                              ; PHYSICAL LAYER STATUS REGISTER 2
;
Symbol	PHSTAT2_TXSTAT = PHSTAT2H.5              ; PHY Transmit Status bit
Symbol	PHSTAT2_RXSTAT = PHSTAT2H.4              ; PHY Receive Status bit
Symbol	PHSTAT2_COLSTAT= PHSTAT2H.3              ; PHY Collision Status bit
Symbol	PHSTAT2_LSTAT  = PHSTAT2H.2              ; PHY Link Status bit (non-latching)
Symbol	PHSTAT2_DPXSTAT= PHSTAT2H.1              ; PHY Duplex Status bit
Symbol	PHSTAT2_PLRITY = PHSTAT2.5               ; Polarity Status bit
;------------------------------------------------------------------------------
;* PHIE bits  
;
;Dim PHIE As Word                                 ; PHY INTERRUPT ENABLE REGISTER
;
;Symbol	PHIE_PLNKIE	   = PHIE.4                  ; PHY Link Change Interrupt Enable bit
;Symbol	PHIE_PGEIE	   = PHIE.1                  ; PHY Global Interrupt Enable bit
;------------------------------------------------------------------------------
;* PHIR bits  
;
;Dim PHIR As Word                                 ; PHY INTERRUPT REQUEST (FLAG) REGISTER
;
;Symbol	PHIR_PLNKIF	   = PHIR.4                  ; PHY Link Change Interrupt Flag bit
;Symbol	PHIR_PGIF	   = PHIR.2                  ; PHY Global Interrupt Flag bit
;------------------------------------------------------------------------------
;* PHLCON bits  
;
Dim PHLCON As Word                               ; PHY MODULE LED CONTROL REGISTER
;
Symbol	PHLCON_LACFG3  = PHLCONH.3               ; LEDA Configuration bits
Symbol	PHLCON_LACFG2  = PHLCONH.2               ; LEDA Configuration bits
Symbol	PHLCON_LACFG1  = PHLCONH.1               ; LEDA Configuration bits
Symbol	PHLCON_LACFG0  = PHLCONH.0               ; LEDA Configuration bits
Symbol	PHLCON_LBCFG3  = PHLCON.7                ; LEDB Configuration bits
Symbol	PHLCON_LBCFG2  = PHLCON.6                ; LEDB Configuration bits
Symbol	PHLCON_LBCFG1  = PHLCON.5                ; LEDB Configuration bits
Symbol	PHLCON_LBCFG0  = PHLCON.4                ; LEDB Configuration bits
Symbol	PHLCON_LFRQ1   = PHLCON.3                ; LED Pulse Stretch Time Configuration bits
Symbol	PHLCON_LFRQ0   = PHLCON.2                ; LED Pulse Stretch Time Configuration bits
Symbol	PHLCON_STRCH   = PHLCON.1                ; LED Pulse Stretching Enable bit
;
;
;
;
;******************************************************************************
;* ENC28J60 Driver - Variables used (declarations)
;******************************************************************************
Dim ETH_TMP_BYTE_1      As Byte                        ; Temp var // Byte type
Dim ETH_TMP_BYTE_2      As Byte                        ; Temp var // Byte type
Dim ETH_TMP_BYTE_3      As Byte                        ; Temp var // Byte type
Dim ETH_TMP_WORD_1      As Word                        ; Temp var // Word type
Dim ETH_TMP_WORD_2      As Word                        ; Temp var // Word type
Dim PHY_TMP_WORD_1      As Word                        ; Temp var // Word type
;
;
; a variable used for testing Macros only, pls remove
Dim DUMB                As Byte
Dim DUMB_BYTE           As Byte
Dim DUMB_WORD           As Word
;******************************************************************************
Dim TX_PRE_BUF[128]     As Byte                        ; Shadow buffer for transmitting packets
Dim FSR2 As FSR2L.Word        						   ; Create a word variable from registers FSR2L\H 

; --- needed vars  for CKSUM ----
Dim CKSUM_PIC_WORD As Word
Dim CKSUM_PIC_DWORD As Dword
Dim CKSUM_PIC_LOOP_WORD As Word
Dim TX_PRE_LOW_PNTR As Word
Dim TX_PRE_HIGH_PNTR As Word
Dim ODD_OR_EVEN As Word
Dim ODD As Byte
Dim INTER_CKSUM As Dword
Dim TEMPG As Word
Dim TEMPGG As Word
;--------------------------------
; UDP vars
;----------------------------
Dim UDP_REPLY_STRING As String * 80    ; we go for a max length of 80 bytes (ie if TX_PRE_BUFFER is 128) for UDP reply string
Dim UDP_REPLY_STRING_LEN As Byte       ; hold the UDP_REPLY_STRING Lenght
;------------------------------------------------------------------------------
;* PIC MSSP Register settings
;                                         
SSPCON1 = $20                                          ; Enable SPI,idle Low, Fosc/4
SSPSTAT = $40                                          ; transmit On rising edge, sample middle
;------------------------------------------------------------------------------
;* ENC28J60 Hardware connection to Host Controller
;
Symbol ETH_INT	        = PORTC.0		               ; ENC28J60 INT Pin           <--
Symbol ETH_WOL          = PORTC.1		               ; ENC28J60 WOL Pin           <--
Symbol ETH_CS	        = PORTC.2	                   ; ENC28J60 SPI ENABLE Pin	-->
Symbol ETH_SCK	        = PORTC.3                      ; ENC28J60 SPI CLOCK  Pin    -->
Symbol ETH_SO	        = PORTC.4	                   ; ENC28J60 SPI Data Out Pin	<--
Symbol ETH_SI	        = PORTC.5	                   ; ENC28J60 SPI Data In Pin	-->
;* Configure PIC pins connected To ENC28J60
Input  ETH_INT                                         ; <--
Input  ETH_WOL                                         ; <--
Output ETH_CS                                          ; -->
Output ETH_SCK                                         ; -->
Input  ETH_SO                                          ; <--
Output ETH_SI                                          ; -->
;------------------------------------------------------------------------------
;* ENC28J60 SRAM definitions 
;
Dim MAXFRAME As Word
       MAXFRAME         = 1518                         ; Maximum size for an accepted frame
Symbol TX_BUFFER_SIZE   = 4096                         ; Set Tx Buffer size . Can increase it if needed (ie big web page)
Symbol TX_BUF_START	    = 8192 - (TX_BUFFER_SIZE + 8)  ; Compute Tx Buffer Start Location
Symbol TX_BUF_END	    = (TX_BUF_START + TX_BUFFER_SIZE) + 7 ; Compute Tx Buffer End Location
Dim    RX_BUF_START     As Word                        ; workaround to have RX_BUF_START declared as Word
       RX_BUF_START	    = 0                            ; Set Rx Buffer Start Location ( should be even )
Symbol RX_BUF_STOP	    = (TX_BUF_START - 1)           ; Compute Rx Buffer Stop Location
Symbol RX_BUFFER_SIZE   = (RX_BUF_STOP-RX_BUF_START+1) ; Compute Rx Buffer Size
;------------------------------------------------------------------------------
;* ENC28J60 SPI Commands Opcodes (to be ORed with a 5 bit register's address)
; 
Symbol	RCR_OP_CODE     = %00000000                     ; Read Control Register command Opcode
Symbol  RBM_OP_CODE     = %00111010                     ; Read Buffer Memory command Opcode
Symbol	WCR_OP_CODE     = %01000000                     ; Write Control Register command Opcode
Symbol	WBM_OP_CODE     = %01111010                     ; Write Buffer Memory command Opcode
Symbol  BFS_OP_CODE     = %10000000                     ; Bit Field Set command Opcode
Symbol	BFC_OP_CODE     = %10100000                     ; Bit Field Clear command  Opcode
Symbol	SRC_OP_CODE     = %11111111                     ; System Command Opcode  
;------------------------------------------------------------------------------
;
;******************************************************************************
;*	HARDWARE (MAC) ADDRESS DEFINITION
;*  ENC28J60 Module MAC hardware address.
;*  Can be changed to any valid address.
;******************************************************************************
Symbol MYMAC_0 = 0                                      ; 1st MAC Address first byte (ie Leftmost) (ie stored in MAADR6)
Symbol MYMAC_1 = 0                                      ; 2nd
Symbol MYMAC_2 = "G"                                    ; 3rd
Symbol MYMAC_3 = "A"                                    ; 4th
Symbol MYMAC_4 = "B"                                    ; 5th
Symbol MYMAC_5 = "I"                                    ; 6th
;------------------------------------------------------------------------------
;*  SOFTWARE (IP) ADDRESS DEFINITION
;*  ENC28J60 Module IP address
;*  Can be changed to any valid address
'******************************************************************************
Symbol MYIP_3  = 192                                    ; <-- leftmost group number
Symbol MYIP_2  = 168                                    ;
Symbol MYIP_1  = 1                                      ;
Symbol MYIP_0  = 8                                      ;
;------------------------------------------------------------------------------
;
;
;
;******************************************************************************
;*  ENC28J60 - Tx and Rx Declarations
;------------------------------------------------------------------------------
Symbol TX_CONTROL_BYTE   = $0F                          ; PER PACKET CONTROL BYTES - See section 7.1 in datasheet
Dim NEW_TX_PACK_PNTR  As Word                          ; POINTER for the next Tx packet start in nic Buffer
;******************************************************************************
;*	Received Packet Buffer Layout 
;******************************************************************************
;*  - The RX packet of 1518 bytes is divided in following arrays in order:
;*    RX_PACKET_HEADER     =  6 bytes
;*    ETH_COM_PACKET       = 74 bytes
;*  - The rest of Ethernet packet --> stored in Reserve RAM Locations (Not available in this version ;-) 
;*  -
;****************************************************************************** 
;*	Receive Packet Buffer Header Layout    - 6 bytes
;******************************************************************************
Dim RX_PACKET_HEADER[6] As Byte                        ; Contains Next Packet Pointer and Receive Status Vector
Dim RX_PACKET_HEADER#0H As RX_PACKET_HEADER#1          ;
Dim NEXT_PACKET_POINTER As RX_PACKET_HEADER#0.Word     ; Next Rx Packet Pointer Value in Rx Buffer
Dim RX_PACKET_HEADER#2H As RX_PACKET_HEADER#3          ;
Dim RX_BYTE_COUNT       As RX_PACKET_HEADER#2.Word     ; Indicates length of the received frame. See section 7.2.2 Table 7-3
Dim RX_PACKET_HEADER#4H As RX_PACKET_HEADER#5          ;
Dim RX_VEC_STATUS       As RX_PACKET_HEADER#4.Word     ; Various Rx Status Vectors. See section 7.2.2 Table 7-3
;******************************************************************************
;*	Common Ethernet Header Layout          - 56 bytes
;******************************************************************************
Dim ETH_COM_PACKET[128]  As Byte                       ; was 56, then changed to 74 for ICMP total bytes
;* so we read 128 bytes from eth packet. If more needed we declare another array and we load it with data....
Dim DEST_MAC_ADR_0      As ETH_COM_PACKET#0            ; Destination MAC Address
Dim DEST_MAC_ADR_1      As ETH_COM_PACKET#1            ;
Dim DEST_MAC_ADR_2      As ETH_COM_PACKET#2            ;
Dim DEST_MAC_ADR_3      As ETH_COM_PACKET#3            ;
Dim DEST_MAC_ADR_4      As ETH_COM_PACKET#4            ;
Dim DEST_MAC_ADR_5      As ETH_COM_PACKET#5            ;
Dim SRC_MAC_ADR_0       As ETH_COM_PACKET#6            ; Source MAC Address
Dim SRC_MAC_ADR_1       As ETH_COM_PACKET#7            ;
Dim SRC_MAC_ADR_2       As ETH_COM_PACKET#8            ;
Dim SRC_MAC_ADR_3       As ETH_COM_PACKET#9            ;
Dim SRC_MAC_ADR_4       As ETH_COM_PACKET#10           ;
Dim SRC_MAC_ADR_5       As ETH_COM_PACKET#11           ;
Dim RX_PACKET_TYPE      As Word                        ;
#define RX_PACKET_TYPEH    ETH_COM_PACKET#12           ; Type of Packet or the Length of the Packet
#define RX_PACKET_TYPE     ETH_COM_PACKET#13           ;

;******************************************************************************
;*  ARP Header Layout aliases to ETH_COM_PACKET (Ethernet Packet) elements
;******************************************************************************
Dim ARP_HW_TYPE         As Word
#define ARP_HW_TYPEH       ETH_COM_PACKET#14           ; ARP Hardware address space in bytes
#define ARP_HW_TYPE        ETH_COM_PACKET#15           ;
Dim ARP_PROT_TYPE       As Word                        ;
#define ARP_PROT_TYPEH     ETH_COM_PACKET#16           ; ARP Protocol address space in bytes
#define ARP_PROT_TYPE      ETH_COM_PACKET#17           ;
Dim ARP_HW_LEN          As ETH_COM_PACKET#18           ; ARP byte length of each hardware address
Dim ARP_PROT_LEN        As ETH_COM_PACKET#19           ; ARP byte length of each protocol address
Dim ARP_OP              As Word                        ;
#define ARP_OPH            ETH_COM_PACKET#20           ; ARP Operations code
#define ARP_OP             ETH_COM_PACKET#21           ;
Dim ARP_SHADDR_0        As ETH_COM_PACKET#22           ; ARP Source MAC Address
Dim ARP_SHADDR_1        As ETH_COM_PACKET#23           ;
Dim ARP_SHADDR_2        As ETH_COM_PACKET#24           ;
Dim ARP_SHADDR_3        As ETH_COM_PACKET#25           ;
Dim ARP_SHADDR_4        As ETH_COM_PACKET#26           ;
Dim ARP_SHADDR_5        As ETH_COM_PACKET#27           ;
Dim ARP_SIPADDR_0       As ETH_COM_PACKET#28           ; ARP Source IP Address
Dim ARP_SIPADDR_1       As ETH_COM_PACKET#29           ; 
Dim ARP_SIPADDR_2       As ETH_COM_PACKET#30           ; 
Dim ARP_SIPADDR_3       As ETH_COM_PACKET#31           ;             
Dim ARP_THADDR_0        As ETH_COM_PACKET#32           ; ARP Target MAC Address
Dim ARP_THADDR_1        As ETH_COM_PACKET#33           ; 
Dim ARP_THADDR_2        As ETH_COM_PACKET#34           ;
Dim ARP_THADDR_3        As ETH_COM_PACKET#35           ;
Dim ARP_THADDR_4        As ETH_COM_PACKET#36           ;
Dim ARP_THADDR_5        As ETH_COM_PACKET#37           ;  
Dim ARP_TIPADDR_0       As ETH_COM_PACKET#38           ; ARP Target IP Address
Dim ARP_TIPADDR_1       As ETH_COM_PACKET#39           ;
Dim ARP_TIPADDR_2       As ETH_COM_PACKET#40           ;
Dim ARP_TIPADDR_3       As ETH_COM_PACKET#41           ;  
;******************************************************************************
;*  IP Header Layout aliases to ETH_COM_PACKET (Ethernet Packet) elements  -  RFC791
;*  - this portion of IP Packet is common to: IP, ICMP, TCP and UDP
;******************************************************************************
Dim IP_VERS_LEN         As ETH_COM_PACKET#14           ; 4 bit VERSION + 4 bit Internet Header Length 
Dim IP_TOS              As ETH_COM_PACKET#15           ; IP type of service
Dim IP_PKT_LEN          As Word                        ;
#define IP_PKT_LENH        ETH_COM_PACKET#16           ; IP Packet Length High Byte
#define IP_PKT_LEN         ETH_COM_PACKET#17           ; IP Packet Length Low Byte                       ;
Dim IP_ID_0             As ETH_COM_PACKET#18           ; IP ID 0   Identification
Dim IP_ID_1             As ETH_COM_PACKET#19           ; IP ID 1        -"-
Dim IP_FRAG_OFFSET_0    As ETH_COM_PACKET#20           ; IP Flags and Fragments Offset 0
Dim IP_FRAG_OFFSET_1    As ETH_COM_PACKET#21           ; IP Flags and Fragments Offset 1
Dim IP_TTL              As ETH_COM_PACKET#22           ; IP Time to Live
Dim IP_PROT             As ETH_COM_PACKET#23           ; IP next level Protocol (ICMP=1, TCP=6, UDP=11)                     
Dim IP_HDR_CKSUM        As Word                        ;
#define IP_HDR_CKSUMH      ETH_COM_PACKET#24           ; IP Header Checksum High Byte
#define IP_HDR_CKSUM       ETH_COM_PACKET#25           ; IP Header Checksum Low Byte          
Dim IP_SRC_ADDR_0       As ETH_COM_PACKET#26           ; IP Source Address (IP)  - leftmost one
Dim IP_SRC_ADDR_1       As ETH_COM_PACKET#27           ;   
Dim IP_SRC_ADDR_2       As ETH_COM_PACKET#28           ;  
Dim IP_SRC_ADDR_3       As ETH_COM_PACKET#29           ;  
Dim IP_DEST_ADDR_0      As ETH_COM_PACKET#30           ; IP Destination Address (IP) - leftmost one
Dim IP_DEST_ADDR_1      As ETH_COM_PACKET#31           ;
Dim IP_DEST_ADDR_2      As ETH_COM_PACKET#32           ;
Dim IP_DEST_ADDR_3      As ETH_COM_PACKET#33           ; From here on IP Data
;******************************************************************************
;*	TCP Header Layout aliases To ETH_COM_PACKET (Ethernet Packet) elements  - RFC793
;******************************************************************************
Dim TCP_SRC_PORT        As Word
#define TCP_SRC_PORTH      ETH_COM_PACKET#34           ; TCP Source Port Number High Byte
#define TCP_SRC_PORT       ETH_COM_PACKET#35           ; TCP Source Port Number Low Byte
Dim TCP_DEST_PORT       As Word
#define TCP_DEST_PORTH     ETH_COM_PACKET#36           ; TCP Destination Port Number High Byte
#define TCP_DEST_PORT      ETH_COM_PACKET#37           ; TCP Destination Port Number Low Byte
;
Dim TCP_SEQ_NUM         As Dword  
#define TCP_SEQ_NUMHHH     ETH_COM_PACKET#38           ; TCP Sequance Number 3
#define TCP_SEQ_NUMHH      ETH_COM_PACKET#39           ; TCP Sequence Number 2
#define TCP_SEQ_NUMH       ETH_COM_PACKET#40           ; TCP Sequence Number 1
#define TCP_SEQ_NUM        ETH_COM_PACKET#41           ; TCP Sequence Number 0
;
Dim TCP_ACK_NUM         As Dword
#define TCP_ACK_NUMHHH     ETH_COM_PACKET#42           ; TCP Acknowledge Number 3
#define TCP_ACK_NUMHH      ETH_COM_PACKET#43           ; TCP Acknowledge Number 2
#define TCP_ACK_NUMH       ETH_COM_PACKET#44           ; TCP Acknowledge Number 1
#define TCP_ACK_NUM        ETH_COM_PACKET#45           ; TCP Acknowledge Number 0
;
Dim TCP_HDR_FLAGS_0     As ETH_COM_PACKET#46           ; TCP HEADER and FLAGS first byte  (4 bit Header)
Dim TCP_HDR_FLAGS_1     As ETH_COM_PACKET#47           ; TCP HEADER and FLAGS second byte (rest Flags)       
; here we dim the flag bits
#define  URG               TCP_HDR_FLAGS_1,5           ; Urgent pointer valid flag
;#DEFINE  ACK               TCP_HDR_FLAGS_1,4           ; Acknowledgment number valid flag. When set indicates that the Acknowledgement Number is being used.
Symbol ACK = TCP_HDR_FLAGS_1.4
#define  PSH               TCP_HDR_FLAGS_1,3           ; Push flag. An upper level protocol requires immediate data delivery
#define  RST               TCP_HDR_FLAGS_1,2           ; Reset connection flag
#define  SYN               TCP_HDR_FLAGS_1,1           ; Synchronize sequence numbers flag. When set, it indicates that this segment is the first one in the sequence
#define  FIN               TCP_HDR_FLAGS_1,0           ; Finish Flag. When set, it indicates that this is the last data from the sender
;
Dim TCP_WINDOW          As Word
#define TCP_WINDOWH        ETH_COM_PACKET#48           ; TCP WINDOW High Byte
#define TCP_WINDOW         ETH_COM_PACKET#49           ; TCP WINDOW Low Byte
Dim TCP_CKSUM           As Word
#define TCP_CKSUMH         ETH_COM_PACKET#50           ; TCP Checksum High Byte
#define TCP_CKSUM          ETH_COM_PACKET#51           ; TCP Checksum Low Byte
Dim TCP_URGENT_PTR      As Word
#define TCP_URGENT_PTRH    ETH_COM_PACKET#52           ; TCP Urgent Pointer High Byte
#define TCP_URGENT_PTR     ETH_COM_PACKET#53           ; TCP Urgent Pointer Low Byte. From here on TCP option/data.
;******************************************************************************
;*	ICMP Header Layout aliases To ETH_COM_PACKET (Ethernet Packet) elements  - RFC792
;*  - Format as per ECHO or REPLY Messages . See RFC792 pag.14
;******************************************************************************
Dim ICMP_TYPE           As ETH_COM_PACKET#34           ; ICMP TYPE
Dim ICMP_CODE           As ETH_COM_PACKET#35           ; ICMP CODE
Dim ICMP_CKSUM          As Word
#define ICMP_CKSUMH        ETH_COM_PACKET#36           ; ICMP CHECKSUM High Byte
#define ICMP_CKSUM         ETH_COM_PACKET#37           ; ICMP CHECKSUM Low Byte
Dim ICMP_IDENT          As Word
#define ICMP_IDENTH        ETH_COM_PACKET#38           ; ICMP IDENTIFIER High Byte
#define ICMP_IDENT         ETH_COM_PACKET#39           ; ICMP IDENTIFIER Low Byte
Dim ICMP_SEQ_NUM        As Word
#define ICMP_SEQ_NUMH      ETH_COM_PACKET#40           ; ICMP SEQUENCE NUMBER High Byte
#define ICMP_SEQ_NUM       ETH_COM_PACKET#41           ; ICMP SEQUENCE NUMBER Low Byte. From here on ICMP data.
;******************************************************************************
;*	UDP Header Layout aliases To ETH_COM_PACKET (Ethernet Packet) elements  - RFC768
;******************************************************************************
Dim UDP_SRC_PORT        As Word
#define UDP_SRC_PORTH      ETH_COM_PACKET#34           ; UDP SOURCE PORT High Byte
#define UDP_SRC_PORT       ETH_COM_PACKET#35           ; UDP SOURCE PORT Low Byte
Dim UDP_DEST_PORT       As Word
#define UDP_DEST_PORTH     ETH_COM_PACKET#36           ; UDP DESTINATION PORT High Byte
#define UDP_DEST_PORT      ETH_COM_PACKET#37           ; UDP DESTINATION PORT High Byte
Dim UDP_LEN             As Word
#define UDP_LENH           ETH_COM_PACKET#38           ; UDP length  in octets  of this user datagram  High Byte
#define UDP_LEN            ETH_COM_PACKET#39           ; UDP length  in octets  of this user datagram  Low Byte
Dim UDP_CKSUM           As Word
#define UDP_CKSUMH         ETH_COM_PACKET#40           ; UDP CHECKSUM High Byte
#define UDP_CKSUM          ETH_COM_PACKET#41           ; UDP CHECKSUM Low Byte. From here on UDP Data.
;******************************************************************************
;*	IP Protocol Types   for test IP Next level Protocol (ICMP=$01, TCP=$06, UDP=$11)
;*  - IP_PROT variable     (i.e. ETH_COM_PACKET#23 )
;******************************************************************************
Symbol	PROT_ICMP		= $01                          ; IP Protocol type - ICMP
Symbol	PROT_TCP		= $06                          ; IP Protocol type - TCP
Symbol	PROT_UDP		= $11                          ; IP Protocol type - EDP
Symbol  ARP_TYPE_HEADER = $0806                        ; ARP type header
Symbol  TCP_TYPE_HEADER = $0800                        ; TCP type header
Symbol  ARP_OP_REQUEST  = $01                          ; it's an ARP Request Operation
Symbol  ARP_OP_REPLY    = $02                          ; it's an ARP Reply Operation
Symbol  ICMP_ECHO_REQUEST = $08                        ; it's an ICMP ECHO Request
Symbol  ICMP_ECHO_REPLY = $00                          ; it's an ICMP ECHO Reply
;******************************************************************************
;
;
;
;
; ENC28J60 - MACRO's Place - HERE
;------------------------------------------------------------------------------
GoTo OVER_ETH_MACROS
;******************************************************************************
;*  SPI Shift-out Shift-in using HARDWARE MSSP - Macro
;******************************************************************************
;*  - argument: value to send (an 8bit constant number or a byte type variable)
;*  - returns inline Byte type VARIABLE (copy of SSPBUF) - if used inline
;******************************************************************************
SPI                Macro P1                       
                   #if (Prm_1 == Byte)
		               Byte_Byte P1, SSPBUF
                   #endif
	               #if (Prm_1 == Num8) | (Prm_1 == Num16)
		               Num_Byte P1, SSPBUF
	               #endif
                   GoSub SPI_SUB
                   #if(SPI_RETURN == 1)
                       #if(Return_Type == Byte)
                           Wreg_Byte Return_Var
                       #else
                           #error "Return Var for SPI Macro is not a Byte type"
                       #endif
                   #endif
                   Endm    
SPI_SUB:
                   Btfss  SSPSTAT.0                             ; Has data been received(transmit complete)? 
                   Bra $ - 2                     '  
                   Byte_Wreg SSPBUF               
                   Return                
;******************************************************************************
;*	CLEAR CONTROL REGISTER BIT(S) - Macro
;******************************************************************************
;*  - arguments: Register Address, Data to be written
;*  - bits to be cleared have to be Set in the Data parameter
;*  - use only for ETH registers 
;******************************************************************************
CLEAR_REGISTER_BIT Macro addr, datas
                   #if (Prm_1 == Num8) || (Prm_1 == Num16)           
                      Num_Word addr, ETH_TMP_WORD_2   
                   #else
                      #error "BFC Reg address --> not a constant number"
                   #endif
                   
                   #if (Prm_2 == Num8)
                      Num_Byte datas, ETH_TMP_BYTE_2
                   #endif
                   #if (Prm_2 == Byte)
                      Byte_Byte datas, ETH_TMP_BYTE_2
                   #endif 
                   #if (Prm_2 != Num8) && (Prm_2 != Byte)
                      #error "BFC Data variable should be a Byte type !"
                   #endif
                   GoSub CLEAR_REGISTER_BIT_SUB
                   Endm
CLEAR_REGISTER_BIT_SUB:
                   Bcf ETH_CS                                  ; Enable SPI on ENC28J60
                   ETH_TMP_BYTE_3 = (BFC_OP_CODE | ETH_TMP_WORD_2.LowByte)  
                   SPI ETH_TMP_BYTE_3                          ; Send first the address of Control Register
                   SPI ETH_TMP_BYTE_2                          ; Then the data byte
                   Bsf ETH_CS                                  ; Disable SPI on ENC28J60
                   Return
;******************************************************************************
;*	SET CONTROL REGISTER BIT(S) - Macro
;******************************************************************************
;*  - arguments: Register Address, Data to be written
;*  - bits to be set have to be Set in the Data parameter
;*  - use only for ETH registers
;******************************************************************************
SET_REGISTER_BIT   Macro addr, datas
                   #if (Prm_1 == Num8) || (Prm_1 == Num16)           
                      Num_Word addr, ETH_TMP_WORD_2   
                   #else
                      #error "BFS Reg address --> not a constant number"
                   #endif
                   #if (Prm_2 == Num8)
                      Num_Byte datas, ETH_TMP_BYTE_2
                   #endif
                   #if (Prm_2 == Byte)
                      Byte_Byte datas, ETH_TMP_BYTE_2
                   #endif 
                   #if (Prm_2 != Num8) && (Prm_2 != Byte)
                      #error "BFS Data variable should be a Byte type !"
                   #endif
                   GoSub SET_REGISTER_BIT_SUB
                   Endm
SET_REGISTER_BIT_SUB:
                   Bcf ETH_CS                                  ; Enable SPI on ENC28J60
                   ETH_TMP_BYTE_3 = (BFS_OP_CODE | ETH_TMP_WORD_2.LowByte)  ; Send first the address of Control Register
                   SPI ETH_TMP_BYTE_3
                   SPI ETH_TMP_BYTE_2                          ; Then the data byte
                   Bsf ETH_CS                                  ; Disable SPI on ENC28J60
                   Return
;*****************************************************************************
;*	READ MAC or MII CONTROL REGISTER - Macro
;*****************************************************************************
;*  - argument : MAC or MII Register Address
;*  - use only for MAC or MII Registers
;*  - returns inline byte type variable
;*****************************************************************************
READ_MAC_REGISTER Macro addr
                  #if (Prm_1 == Num8) || (Prm_1 == Num16)           
                      Num_Word addr, ETH_TMP_WORD_1   
                  #else
                      #error "MAC Reg address --> not a constant number"
                  #endif
                  GoSub READ_MAC_REGISTER_SUB
                  #if(READ_MAC_REGISTER_RETURN == 1)          ; if a Return Value Requested
                      #if(Return_Type == Byte)                ; if the Inline var is a byte type
                           ;;WREG_BYTE RETURN_VAR
                           Byte_Byte SSPBUF,Return_Var
                      #else
                           #error "Return Var for READ_MAC_REGISTER --> not a Byte type"
                      #endif
                  #endif           
                  Endm
READ_MAC_REGISTER_SUB:                                                     
                  CLEAR_REGISTER_BIT ADR_ECON1,%00000011
                  SET_REGISTER_BIT ADR_ECON1, ETH_TMP_WORD_1.HighByte
                  ETH_TMP_BYTE_3 = (RCR_OP_CODE | ETH_TMP_WORD_1.LowByte) 
                  Bcf ETH_CS                                  ; Enable SPI on ENC28J60
                  SPI ETH_TMP_BYTE_3                          
                  SPI $FF                                     ; SSPBUF will hold a dummy value (if MAC or MII)
                  SPI $FF                                     ; OK, now we got the MAC or MII Control Register value in SSPBUF 
                  ;;BYTE_WREG SSPBUF                            ; make a copy of SSPBUF in WREG
                  Bsf ETH_CS                                  ; Disable SPI on ENC28J60  
                  Return
;******************************************************************************
;*	READ ETH CONTROL REGISTER - Macro
;******************************************************************************
;*  - argument : ETH Register Address
;*  - use only for ETH Registers
;*  - returns inline byte type variable
;******************************************************************************
READ_ETH_REGISTER Macro addr
                  #if (Prm_1 == Num8) || (Prm_1 == Num16)           
                      Num_Word addr, ETH_TMP_WORD_1   
                  #else
                      #error "ETH Reg address --> not a constant number"
                  #endif
                  GoSub READ_ETH_REGISTER_SUB
                  #if(READ_ETH_REGISTER_RETURN == 1)          ; if a Return Value Requested
                      #if(Return_Type == Byte)                ; if the Inline var is a byte type
                           ;;WREG_BYTE RETURN_VAR
                           Byte_Byte SSPBUF,Return_Var
                      #else
                           #error "Return Var for READ_ETH_REGISTER --> not a Byte type"
                      #endif
                  #endif           
                  Endm
READ_ETH_REGISTER_SUB:
                  ;bcf ETH_CS                                                     
                  CLEAR_REGISTER_BIT ADR_ECON1,%00000011
                  SET_REGISTER_BIT ADR_ECON1, ETH_TMP_WORD_1.HighByte
                  ETH_TMP_BYTE_3 = (RCR_OP_CODE | ETH_TMP_WORD_1.LowByte)  
                  Bcf ETH_CS                                ; Enable SPI on ENC28J60 
                  SPI ETH_TMP_BYTE_3 
                  SPI $FF                                   ; SSPBUF will hold the value of ETH Control Register
                  ;;BYTE_WREG SSPBUF                            ; make a copy of SSPBUF in WREG
                  Bsf ETH_CS                                  ; Disable SPI on ENC28J60 
                  Return
;******************************************************************************
;*	WRITE MAC AND ETH CONTROL REGISTER - Macro
;******************************************************************************
;*  - arguments: Register Address , Data to be written
;*  - used for written to ETH or MAC MII registers
;******************************************************************************
WRITE_ETH_REGISTER Macro addr, datas
                   #if (Prm_1 == Num8) || (Prm_1 == Num16)           
                      Num_Word addr, ETH_TMP_WORD_1
                   #else
                      #error "ETH Reg address --> not a constant number"
                   #endif
                   ;;
                   #if (Prm_2 == Num8) || (Prm_2 == Num16)
                      Num_Byte datas, ETH_TMP_BYTE_1
                   #endif
                   #if (Prm_2 == Byte)
                      Byte_Byte datas, ETH_TMP_BYTE_1
                   #endif 
                   ;*********
                   #if (Prm_2 == Word)
                      Word_Byte datas, ETH_TMP_BYTE_1
                   #endif 
                   ;*********
                   #if (Prm_2 != Num8) && (Prm_2 != Byte) && (Prm_2 != Word) && (Prm_2 != Num16)
                      #error "WCR Data variable not a supported type !"
                   #endif
                   GoSub WRITE_ETH_REGISTER_SUB
                   Endm
WRITE_ETH_REGISTER_SUB:                                 
                   CLEAR_REGISTER_BIT ADR_ECON1,%00000011
                   SET_REGISTER_BIT ADR_ECON1, ETH_TMP_WORD_1.HighByte
                   ETH_TMP_BYTE_3 = (WCR_OP_CODE | ETH_TMP_WORD_1.LowByte)  ; Send first the address of Control Register
                   Bcf ETH_CS                                  ; Enable SPI on ENC28J60
                   SPI ETH_TMP_BYTE_3
                   SPI ETH_TMP_BYTE_1                          ; Then the data byte
                   Bsf ETH_CS                                  ; Disable SPI on ENC28J60
                   Return
;******************************************************************************
;*	ENC28J60 SOFTWARE RESET ROUTINE - Macro
;******************************************************************************
;*  -
;*  _
;******************************************************************************
NIC_SOFT_RESET     Macro
                   Bcf ETH_CS                                  ; Enable SPI on ENC28J60
                   SPI SRC_OP_CODE
                   Bsf ETH_CS                                  ; Disable SPI on ENC28J60
                   Endm
;******************************************************************************
;*	READ ENC28J60 PHY REGISTER - Macro
;******************************************************************************
;*  - argument: PHY Register address (byte type , actually a numeric constant)
;*  - returns Word type value for Inline uses, or
;*  - returns the PHY Reg Value in PHY_TMP_WORD_1 - word type variable
;******************************************************************************
READ_PHY_REGISTER  Macro addr
                   #if (Prm_1 == Num8)             
                      Num_Byte addr, ETH_TMP_BYTE_1   
                   #else
                      #error "PHY Reg address --> not a constant number"
                   #endif                                                               
                   GoSub READ_PHY_REGISTER_SUB
                   #if(READ_PHY_REGISTER_RETURN == 1)                   ; if a Return Value Requested
                      #if(Return_Type == Word)                          ; if the Inline var is a byte type
                           GoSub READ_MIRD_SUB
                           Word_Word PHY_TMP_WORD_1, Return_Var
                      #else
                           #error "Return Var for READ_PHY_REGISTER --> not a Word type"
                      #endif
                   #else
                      GoSub READ_MIRD_SUB    
                   #endif
                   Endm
READ_PHY_REGISTER_SUB:                   
                   WRITE_ETH_REGISTER ADR_MIREGADR, ETH_TMP_BYTE_1      ;
                   Clrf MICMD
                   Bsf MICMD_MIIRD
                   WRITE_ETH_REGISTER ADR_MICMD, MICMD
                   DelayUS 20                                           ; wait 10.24us
                   Repeat                                               ; poll MISTAT_BUSY for Clear
                         MISTAT = READ_MAC_REGISTER ADR_MISTAT
                         DelayMS 10
                   Until MISTAT_BUSY = 0
                   Bcf MICMD_MIIRD
                   WRITE_ETH_REGISTER ADR_MICMD, MICMD
                   Return
READ_MIRD_SUB:                   
                   PHY_TMP_WORD_1.LowByte  =  READ_MAC_REGISTER ADR_MIRDL
                   PHY_TMP_WORD_1.HighByte =  READ_MAC_REGISTER ADR_MIRDH
                   Return
;******************************************************************************
;*	WRITE ENC28J60 PHY REGISTER - Macro
;******************************************************************************
;*  - arguments: PHY Register address , 16bit data to be written (like a 16bit constant or Word var)
;*  - returns nothing
;******************************************************************************
WRITE_PHY_REGISTER Macro addr, datas
                   #if (Prm_1 == Num8)            
                      Num_Byte addr, ETH_TMP_BYTE_1
                   #else
                      #error "PHY Reg address --> not an 8bit constant number"
                   #endif
                   
                   #if (Prm_2 == Num16)
                      Num_Word datas, PHY_TMP_WORD_1
                   #endif
                   #if (Prm_2 == Word)
                      Word_Word datas, PHY_TMP_WORD_1
                   #endif
                   #if (Prm_2 != Num16) && (Prm_2 != Word)
                      #error "PHY Register value should be an 16bit type"
                   #endif
                   GoSub WRITE_PHY_REGISTER_SUB
                   Endm
WRITE_PHY_REGISTER_SUB:
                   WRITE_ETH_REGISTER ADR_MIREGADR, ETH_TMP_BYTE_1        ;  (1)
                   WRITE_ETH_REGISTER ADR_MIWRL, PHY_TMP_WORD_1.LowByte   ;  (2)
                   WRITE_ETH_REGISTER ADR_MIWRH,PHY_TMP_WORD_1.HighByte  ;  (3)
                   DelayUS 20                                            ; wait 10.24us
                   Repeat
                         MISTAT = READ_MAC_REGISTER ADR_MISTAT            ; poll MISTAT_BUSY for Clear
                   Until MISTAT_BUSY = 0
                   Return
;******************************************************************************
;*	READ ENC28J60 8K BUFFER MEMORY - Macro
;******************************************************************************
;*  - Read Buffer Pointer is --> ERDPT
;*  - If the AUTOINC bit in the ECON2 register is set, the
;*    ERDPT Pointer will automatically increment to point to
;*    the next address after the last bit of each byte is read.
;*    See: ENC28J60 datasheet section 4.2.2 READ BUFFER MEMORY COMMAND
;*  - returns inline byte type variable
;*
;*  - Alternatively, if you want to continuously Read from ENC28J60 Memory Buffer,
;*    in your routine you can issue something like:
;*    =======================
;*    bcf ETH_CS
;*    for var= 0 to whatever
;*        SPI RBM_OP_CODE   ; read value from memory will be now in SSPBUF
;*        ' store the value of SSPBUF in some array or whatever use
;*    next    
;*    bsf ETH_CS
;*    =======================
;******************************************************************************
READ_BUFFER_MEMORY Macro
                   GoSub READ_BUFFER_MEMORY_SUB
                   #if(READ_BUFFER_MEMORY_RETURN == 1)         ; if a Return Value Requested
                      #if(Return_Type == Byte)                 ; if the Inline var is a byte type
                           ;;WREG_BYTE RETURN_VAR
                           Byte_Byte SSPBUF,Return_Var
                      #else
                           #error "Return Var for READ_BUFFER_MEMORY --> not a Byte type"
                      #endif
                   #endif 
                   Endm
READ_BUFFER_MEMORY_SUB:
                   Bcf ETH_CS                                  ; Enable SPI on ENC28J60
                   SPI RBM_OP_CODE                             ; Send RBM Opcode on SPI 
                   SPI $FF
                   Bsf ETH_CS                                  ; Disable SPI on ENC28J60
                   ;;BYTE_WREG SSPBUF                            ; SSPBUF will hold the value. ?make a copy of SSPBUF in WREG?
                   Return
;******************************************************************************
;*	WRITE ENC28J60 8K BUFFER MEMORY - Macro
;******************************************************************************
;*  - Write Buffer Pointer is --> EWRPT
;*  - If the AUTOINC bit in the ECON2 register is set, after
;*    the last bit of each byte is written, the EWRPT Pointer
;*    will automatically be incremented to point to the next
;*    sequential address (current address + 1).
;*    See: ENC28J60 datasheet section 4.2.4 WRITE BUFFER MEMORY COMMAND
;*
;*  - Alternatively, if you want to continuously Write to ENC28J60 Memory Buffer,
;*    in your routine you can issue something like:
;*    =======================
;*    bcf ETH_CS
;*    SPI WBM_OP_CODE
;*    for var= 0 to whatever
;*        value_to_be_written =  whatever_byte     'update value_to_be_written (byte type)
;*        SPI value_to_be_written   
;*    next    
;*    bsf ETH_CS
;*    =======================
;*
;*  - argument: data value to be written to buffer (byte type)
;*  - 
;******************************************************************************
WRITE_BUFFER_MEMORY Macro datas
                    #if (Prm_1 == Num8)            
                      Num_Byte datas, ETH_TMP_BYTE_1   
                    #endif
                    #if (Prm_1 == Byte)
                      Byte_Byte datas, ETH_TMP_BYTE_1
                    #endif
                    #if (Prm_1 != Num8) && (Prm_1 != Byte)
                        #error "WBM command Macro parameter should be an 8bit value number constant or Byte type"
                    #endif
                    GoSub WRITE_BUFFER_MEMORY_SUB
                    Endm 
WRITE_BUFFER_MEMORY_SUB:
                    Bcf ETH_CS                                  ; Enable SPI on ENC28J60
                    SPI WBM_OP_CODE                             ; Send WBM Opcode on SPI
                    SPI ETH_TMP_BYTE_1                          ; Send the value
                    Bsf ETH_CS                                  ; Disable SPI on ENC28J60
                    Return                 
;******************************************************************************
;*	INITIALIZE ENC28J60 - Macro
;******************************************************************************
INIT_ENC28J60       Macro                   
                    GoSub INIT_ENC28J60_SUB
                    Endm
INIT_ENC28J60_SUB:
                    Bsf ETH_CS                                  ; start with SPI disabled
                    Bcf ESTAT_CLKRDY                            ; clear ESTAT_CLKRDY bit so we can test it's value
                    While ESTAT_CLKRDY = 0                      ; Loop until ESTAT_CLKRDY = 1
                          ESTAT = READ_ETH_REGISTER ADR_ESTAT   ; so we know that the PHY module is ready
                    Wend                                        ;
                    NIC_SOFT_RESET                              ; ENC28J60 need a Software Reset here
                    DelayMS 10                                  ; 10ms delay after Software Reset as per datasheet     
                    Clrf ECON1                                  ; Clear the ECON1 register (variable .. copy) 
                    Bsf ECON1_RXEN                              ; as we need to clear RXEN, TXRST, BSEL0 and BSEL1 bits,
                    Bsf ECON1_TXRST                             ; we need to pass them as "set" to CLEAR_REGISTER_BIT Macro
                    Bsf ECON1_BSEL0                             ;
                    Bsf ECON1_BSEL1                             ;
                    CLEAR_REGISTER_BIT ADR_ECON1, ECON1         ; All Packets Rx'ed ignored and Tx Logic is Idle for now
                    NEXT_PACKET_POINTER = RX_BUF_START          ; Init next Rx Packet Buf Pointer to Rx Buffer Start Location
                    ;;;
                    ; RX Buffer Init
                    WRITE_ETH_REGISTER ADR_ERDPTL,RX_BUF_START  ; Init Hardware Rx Buffer Pointer to the start of  
                    WRITE_ETH_REGISTER ADR_ERDPTH,RX_BUF_STARTH ; declared Rx Buffer value
                    WRITE_ETH_REGISTER ADR_ERXSTL,RX_BUF_START  ; Init Hardware RX Buffer START Location as per declares
                    WRITE_ETH_REGISTER ADR_ERXSTH,RX_BUF_STARTH ;
                    WRITE_ETH_REGISTER ADR_ERXRDPTL,RX_BUF_START  ; see section 7.2.4 "Freeing Receive buffer Space"
                    WRITE_ETH_REGISTER ADR_ERXRDPTH,RX_BUF_STARTH ;
                    WRITE_ETH_REGISTER ADR_ERXNDL,RX_BUF_STOP   ; Init Hardware Rx Buffer END Location as per declares
                    WRITE_ETH_REGISTER ADR_ERXNDH,RX_BUF_STOPH  ;
                    ; TX Buffer Init
                    WRITE_ETH_REGISTER ADR_ETXSTL,TX_BUF_START  ; Init Hardware TX Buffer START Location as per declares
                    WRITE_ETH_REGISTER ADR_ETXSTH,TX_BUF_STARTH ; 
                    ; MAC Init
                    Clrf MACON1                                 ; start with a clear MACON1 as we need to set some bits
                    Bsf MACON1_MARXEN                           ; Enable packets to be received by the MAC
                    Bsf MACON1_RXPAUS                           ; Inhibit transmissions when pause control frames are received 
                    Bsf MACON1_TXPAUS                           ; Allow the MAC to transmit pause control frames
                    WRITE_ETH_REGISTER ADR_MACON1,MACON1        ; Init MACON1 register
	                ;WRITE_ETH_REGISTER ADR_MACON2,0            ; Init MACON2 -- no more in new datasheet
                    Clrf MACON3                                 ; start with a Clear MACON3 As we need To Set some bits
	                Bsf MACON3_FRMLNEN                          ; Frame Length Checking Enable
                    Bsf MACON3_TXCRCEN                          ; Transmit CRC Enable
                    Bsf MACON3_PADCFG0                          ; All short frames zero padded to 60 bytes, and valid CRC appended
	                ; this next for duplex
	                Bsf MACON3_FULDPX
	                ;
                    Bsf MACON3_PADCFG1                         ; change to suit your application need
                    Bsf MACON3_PADCFG2                         ;
                    WRITE_ETH_REGISTER ADR_MACON3,MACON3        ; Init MACON3 register
                    ;clrf MACON4                                 ; start with a clear MACON4 as we need to set some bits
                    ;bsf MACON4_NOBKOFF                          ; No Backoff Enable bit (applies to half duplex only)
                    ;bsf MACON4_DEFER                            ; Defer Transmission Enable bit (applies to half duplex only)
                    ;WRITE_ETH_REGISTER ADR_MACON4,MACON4        ; Init MACON4 register
                    WRITE_ETH_REGISTER ADR_MAIPGL, $12          ; Configure the Non-Back-to-Back Inter-Packet GAP register
	                WRITE_ETH_REGISTER ADR_MAIPGH, $0C          ; For half duplex usage
	                WRITE_ETH_REGISTER ADR_MAMXFLL,MAXFRAME     ; Init the maximum permited frame lentgh for RX or TX
	                WRITE_ETH_REGISTER ADR_MAMXFLH,MAXFRAMEH    ;
	                WRITE_ETH_REGISTER ADR_MABBIPG, $15         ; Init MABBIPG with $15 for full-duplex
	                ; MY MAC ADDRESS
	                WRITE_ETH_REGISTER ADR_MAADR1, MYMAC_5      ; Init ENC28J60 board MAC Address
	                WRITE_ETH_REGISTER ADR_MAADR2, MYMAC_4      ;
	                WRITE_ETH_REGISTER ADR_MAADR3, MYMAC_3      ;
	                WRITE_ETH_REGISTER ADR_MAADR4, MYMAC_2      ;
	                WRITE_ETH_REGISTER ADR_MAADR5, MYMAC_1      ;
	                WRITE_ETH_REGISTER ADR_MAADR6, MYMAC_0      ; 
                    ; PHY Init   See section 6.6 on half or full-duplex. Also see sections 9.1 And 9.2
	                Clrf PHCON2                                 ; start with a Clear PHCON2 As we need To Set some bits
	                Clrf PHCON2H                                ;
                    Bsf PHCON2_HDLDIS                           ; PHY Half-Duplex Loopback Disable
                    WRITE_PHY_REGISTER ADR_PHCON2, PHCON2       ; Init PHCON2 register
                    Clrf PHCON1                                 ; start with a Clear PHCON1 As we need To Set some bits
                    Clrf PHCON1H                                ; see section 6.6 On half Or full-duplex
                    ; this next for duplex
                    Bsf PHCON1_PDPXMD
                    ;
                    WRITE_PHY_REGISTER ADR_PHCON1, PHCON1       ; and section 11.5 PHY Subsystem Reset. 
	                ; PHLCON - LED Status Init
                    Clrf PHLCON                                 ; start with a Clear PHLCON As we need To Set some bits
                    Clrf PHLCONH                                ;
                    ; LED Settings
                    ; Blink fast LEDA
                    Bcf PHLCON_LACFG3                           ; 0
                    Bsf PHLCON_LACFG2                           ; 1
                    Bsf PHLCON_LACFG1                           ; 1    
                    Bsf PHLCON_LACFG0                           ; 1     
                    ; Display Rx/Tx activity on Link
                    Bsf PHLCON_LBCFG3                           ; 1 display Link Status + Tx and Rx activity On LED B
                    Bsf PHLCON_LBCFG2                           ; 1
                    Bsf PHLCON_LBCFG1                           ; 1     - " -
                    Bcf PHLCON_LBCFG0                           ; 0     - " -
                    ;
                    Bsf PHLCON_STRCH                            ; LED Pulse Stretching Enable
                    WRITE_PHY_REGISTER ADR_PHLCON, PHLCON       ; Init PHLCON register
                    ; Enable Packet reception
                    Clrf ECON1                                  ; Clear the ECON1 register (variable .. copy)
                    Bsf ECON1_RXEN                              ; Normal reception enabled
                    SET_REGISTER_BIT ADR_ECON1, ECON1           ; Packet Reception Enabled from here on
                    Bsf ETH_CS                                  ; disable SPI
                    NEW_TX_PACK_PNTR = TX_BUF_START             ; New Tx Packet Pointer in Buffer = Tx Buffer Start, at Init
                    ; 
                    Clrf ECON2
                    Bsf ECON2_AUTOINC 
                    SET_REGISTER_BIT ADR_ECON2, ECON2           ; set the auto increment bit in ECON2
                    Clrf ERXFCON       
                    WRITE_ETH_REGISTER ADR_ERXFCON,ERXFCON
                    Return

;******************************************************************************
;*  CALCULATE CHECKSUM - Macro
;******************************************************************************
;*  - arguments Low_Pointer and High_pointer of the needed bytes to be check-summed
;*  - The bytes should be previously written in TX_PRE_BUFF space. For the first byte in buffer the pointer = 0
;*    using indirect addressing to RAM Reserved space, as in declare
;*  - if the number of bytes for checksum is odd, there will be a "zero" byte padded at the end
;*  - Returns with checksum value in CKSUM_PIC_WORD variable
;****************************************************************************** 
CALC_CKSUM_PIC      Macro low_pointer, high_pointer
                    #if (Prm_1 == Num8) || (Prm_1 == Num16)
                        Num_Word low_pointer, TX_PRE_LOW_PNTR
                    #endif
                    #if (Prm_2 == Num8) || (Prm_2 == Num16)
                        Num_Word high_pointer, TX_PRE_HIGH_PNTR
                    #endif
                    #if (Prm_1 == Byte)
                        Byte_Word low_pointer, TX_PRE_LOW_PNTR
                    #endif
                    #if (Prm_2 == Byte)
                        Byte_Word high_pointer, TX_PRE_HIGH_PNTR
                    #endif
                    #if (Prm_1 == Word)
                        Word_Word low_pointer, TX_PRE_LOW_PNTR
                    #endif
                    #if (Prm_2 == Word)
                        Word_Word high_pointer, TX_PRE_HIGH_PNTR
                    #endif
                    GoSub CALC_CKSUM_PIC_SUB
                    Endm
CALC_CKSUM_PIC_SUB:                                                          ; calculate the checksum in PIC
                    Clear CKSUM_PIC_DWORD                                    ; start with clear cksum_pic_dword
                    Clear ODD_OR_EVEN
                    Clear ODD
                    FSR2 = VarPtr TX_PRE_BUF 
                    FSR2 = FSR2 + TX_PRE_LOW_PNTR
                    DUMB_WORD = (TX_PRE_HIGH_PNTR - TX_PRE_LOW_PNTR + 1)  ; actual bytes number between pointers
                    If DUMB_WORD // 2 != 0 Then
                       ODD = 1
                       DUMB_WORD = (DUMB_WORD / 2) + 1
                    Else 
                         DUMB_WORD = DUMB_WORD / 2
                    EndIf
                    For CKSUM_PIC_LOOP_WORD = 1 To DUMB_WORD
                       CKSUM_PIC_WORD.HighByte = POSTINC2
                       Inc ODD_OR_EVEN
                       If ODD_OR_EVEN = (TX_PRE_HIGH_PNTR - TX_PRE_LOW_PNTR + 1) And ODD = 1 Then
                          CKSUM_PIC_WORD.LowByte = 0                         ; this is the padded "zero" byte
                       Else
                          CKSUM_PIC_WORD.LowByte  = POSTINC2
                       EndIf
                       Inc ODD_OR_EVEN
                       CKSUM_PIC_DWORD = CKSUM_PIC_DWORD + CKSUM_PIC_WORD    ; add value to a dword var
                    Next     
                    CKSUM_PIC_WORD = CKSUM_PIC_DWORD.Word1 + CKSUM_PIC_DWORD.Word0
                    CKSUM_PIC_WORD = ~ CKSUM_PIC_WORD                        ; and here's our cksum
                    Return
;******************************************************************************
ARP_REPLY Macro
          GoSub ARP_REPLY_SUB
          Endm
ARP_REPLY_SUB:
          GoSub WRITE_ETH_TX_HEADER
          Bcf ETH_CS                                  ; Enable SPI on ENC28J60
          SPI WBM_OP_CODE                             ; Send WBM Opcode on SPI as we want to WRITE_BUFFER_MEMORY
          SPI ARP_HW_TYPE.HighByte  ;
          SPI ARP_HW_TYPE.LowByte
          SPI ARP_PROT_TYPE.HighByte
          SPI ARP_PROT_TYPE.LowByte
          SPI ARP_HW_LEN
          SPI ARP_PROT_LEN
          SPI $00  ; this is --> ARP operation High Byte = Reply
          SPI $02  ; this is --> ARP operation LOW Byte = Reply
          SPI MYMAC_0  ; here starts the Source MAC, which is this board's MAC
          SPI MYMAC_1
          SPI MYMAC_2
          SPI MYMAC_3
          SPI MYMAC_4
          SPI MYMAC_5
          SPI MYIP_3   ; here starts the Source IP address, that is this board's IP
          SPI MYIP_2
          SPI MYIP_1
          SPI MYIP_0
          SPI ARP_SHADDR_0 ; here starts the Target MAC , ie the MAC from where the Request was comming
          SPI ARP_SHADDR_1
          SPI ARP_SHADDR_2
          SPI ARP_SHADDR_3
          SPI ARP_SHADDR_4
          SPI ARP_SHADDR_5
          SPI ARP_SIPADDR_0 ; here starts the Target IP
          SPI ARP_SIPADDR_1
          SPI ARP_SIPADDR_2
          SPI ARP_SIPADDR_3
          Bsf ETH_CS
          ' a total of 42 bytes including : per packet byte + ethernet header (14 bytes) 
          NEW_TX_PACK_PNTR =  NEW_TX_PACK_PNTR + 42  ;* ?
          GoSub WRITE_ETH_TX_TAIL
          Return 
;******************************************************************************
ICMP_REPLY Macro
          GoSub ICMP_REPLY_SUB
          Endm 
ICMP_REPLY_SUB:
          GoSub WRITE_ETH_TX_HEADER
          Bcf ETH_CS                                  ; Enable SPI on ENC28J60
          SPI WBM_OP_CODE                             ; Send WBM Opcode on SPI 
          SPI IP_VERS_LEN ; IP version and lenght
          SPI IP_TOS      ; IP TOS
          SPI IP_PKT_LEN.HighByte
          SPI IP_PKT_LEN.LowByte
          SPI IP_ID_0  ; have to check if those two bytes need changing
          SPI IP_ID_1  ;
          SPI IP_FRAG_OFFSET_0  ; have To check If those two bytes need changing
          SPI IP_FRAG_OFFSET_1  ;
          SPI IP_TTL   ;  TTL
          SPI IP_PROT  ;  this should be ICMP Protocol
          ;
          ; now the IP Header checksum
          ; *** or more simple --> just use the received checksum.its the same ***
          ;
          SPI IP_HDR_CKSUM.HighByte ; that is the easy way
          SPI IP_HDR_CKSUM.LowByte  ;
          ;
          SPI MYIP_3 ; source IP
          SPI MYIP_2 ;
          SPI MYIP_1 ;
          SPI MYIP_0 ;
          SPI IP_SRC_ADDR_0 ; we swap the received source IP to destination
          SPI IP_SRC_ADDR_1
          SPI IP_SRC_ADDR_2
          SPI IP_SRC_ADDR_3
          ;till here was the IP Header 
          SPI $0    ; ICMP type , ie $0 = ECHO Reply
          SPI ICMP_CODE
          ; work around to reuse requester chksum
          ICMP_CKSUM = ICMP_CKSUM + $0800
          ;
          SPI ICMP_CKSUM.HighByte ; I will pass it directly to see what ... too lazy to do dma checksum
          SPI ICMP_CKSUM.LowByte  ;
          SPI ICMP_IDENT.HighByte ;
          SPI ICMP_IDENT.LowByte  ;
          SPI ICMP_SEQ_NUM.HighByte ;
          SPI ICMP_SEQ_NUM.LowByte
          For DUMB = 42 To (RX_BYTE_COUNT - 1) ;was 73....42 means the location where the ICMP data starts. ie ETH_COM_PACKET#42
              SPI ETH_COM_PACKET[DUMB]  ; just copy those 32 bytes from last rx IP ICMP packet
          Next
          Bsf ETH_CS
          ' a total of 74 bytes including : per packet byte + ethernet header (14 bytes)
          NEW_TX_PACK_PNTR =  NEW_TX_PACK_PNTR + RX_BYTE_COUNT ; was 74 ;* ?
          GoSub WRITE_ETH_TX_TAIL
          Return          
;******************************************************************************          
UDP_REPLY Macro
          GoSub UDP_REPLY_SUB
          Endm
UDP_REPLY_SUB:       ; in this demo we reply To UDP data on any Port As proof of concept

          ;***************************************************************************
          ;* received UDP payload data starts on ETH_COM_PACKET(42)
          ;* so we can try a small example
          ;***************************************************************************
          Select ETH_COM_PACKET#42      ; check against the pointed received char 
          Case $46                      ; Client sent a "F" character over UDP to our IP 
               UDP_REPLY_STRING = "You have sent 'F' to ENC28J60 Board"
                     
          Case $47                      ; Client sent a "G" character over UDP to our IP
               UDP_REPLY_STRING = "You have sent 'G' to ENC28J60 Board."
               
          Case Else                     ; Client sent anything else over UDP to our IP
               UDP_REPLY_STRING = "Your command was not recognized"
          EndSelect
          ;  
          ;*  Those are the strings that will be sent back to client UDP application.
          ;*  Following the example above, the strings can be arrays as well
          ;*  Also you can Alias a structure starting with ETH_COM_PACKET(42) onwards,
          ;*  so you can check incoming UDP data on that structure 
          ;*  ( not like a single char as in above example)
          ;*
          ;***************************************************************************
             

          ; have to compute now the payload length in bytes
          UDP_REPLY_STRING_LEN = Len (UDP_REPLY_STRING)  ; or we can calculate it
          ; now calculate the IP_PKT_LEN
          IP_PKT_LEN = 20 + 8 + UDP_REPLY_STRING_LEN  ; 20 is the IP header, 8 is the UDP header
          ; prepare the packet in FSR
          FSR2 = VarPtr TX_PRE_BUF   ; point at the start of our virtual buffer
          ; start with common style eth frame header
          POSTINC2 = TX_CONTROL_BYTE ; loc 0  in our virtual buffer
          POSTINC2 = SRC_MAC_ADR_0   ; loc 1
          POSTINC2 = SRC_MAC_ADR_1   ; loc 2
          POSTINC2 = SRC_MAC_ADR_2   ; loc 3
          POSTINC2 = SRC_MAC_ADR_3   ; loc 4
          POSTINC2 = SRC_MAC_ADR_4   ; loc 5
          POSTINC2 = SRC_MAC_ADR_5   ; loc 6
          POSTINC2 = MYMAC_0         ; loc 7
          POSTINC2 = MYMAC_1         ; loc 8
          POSTINC2 = MYMAC_2         ; loc 9
          POSTINC2 = MYMAC_3         ; loc 10
          POSTINC2 = MYMAC_4         ; loc 11
          POSTINC2 = MYMAC_5         ; loc 12
          POSTINC2 = RX_PACKET_TYPE.HighByte ; loc 13
          POSTINC2 = RX_PACKET_TYPE.LowByte  ; loc 14
          ; we start now to write the IP header
          For DUMB = 14 To 23                   ; here we write the first part IP header
              POSTINC2 = ETH_COM_PACKET[DUMB]   ;  Loc 15......
          Next                                  ;  Loc 24......
          ; now zero the checksum place holders as we need to calculate the checksum
          POSTINC2 = 0 ; loc 25   -- this will be IP_HDR_CKSUM.Highbyte
          POSTINC2 = 0 ; loc 26   -- this will be IP_HDR_CKSUM.Lowbyte
          ; now the ending part of IP header
          POSTINC2 = MYIP_3 ; loc 27
          POSTINC2 = MYIP_2 ; loc 28
          POSTINC2 = MYIP_1 ; loc 29
          POSTINC2 = MYIP_0 ; loc 30
          POSTINC2 = IP_SRC_ADDR_0 ; loc 31
          POSTINC2 = IP_SRC_ADDR_1 ; loc 32
          POSTINC2 = IP_SRC_ADDR_2 ; loc 33
          POSTINC2 = IP_SRC_ADDR_3 ; loc 34
          ; now we can calculate the IP checksum
          CALC_CKSUM_PIC 15,34 ; we have now the IP CKSUM in  CKSUM_PIC_WORD
          FSR2 = VarPtr TX_PRE_BUF
          FSR2 = FSR2 + 25  ;point at IP_HDR_CKSUM.Highbyte
          POSTINC2 = CKSUM_PIC_WORD.HighByte    ; we write now the calculated checksum at their right location
          POSTINC2 = CKSUM_PIC_WORD.LowByte     ; in IP header
          ; we will write now the UDP header
          FSR2 = VarPtr TX_PRE_BUF
          FSR2 = FSR2 + 35
          
          ; Set the UDP destination port here
          UDP_DEST_PORT = 30303                    ; set it as you need 
          ;
          
          POSTINC2 = UDP_DEST_PORT.HighByte  ; loc 35 -- we swap UDP source/destination ports (ie if we accept that port,else change code)
          POSTINC2 = UDP_DEST_PORT.LowByte   ; loc 36
          POSTINC2 = UDP_SRC_PORT.HighByte   ; loc 37
          POSTINC2 = UDP_SRC_PORT.LowByte    ; loc 38
          ; now UDP Length
          UDP_LEN = 8 +  UDP_REPLY_STRING_LEN ; 8 is the UDP header in bytes + length of the payload
          POSTINC2 = UDP_LEN.HighByte        ; loc 39
          POSTINC2 = UDP_LEN.LowByte         ; loc 40
          POSTINC2 = 0                       ; loc 41 -- UDP_CKSUM.highbyte , we write zero as we want to compute them
          POSTINC2 = 0                       ; loc 42 -- UDP_CKSUM.lowbyte 
          ;****
          CALC_CKSUM_PIC 27,42
          INTER_CKSUM = CKSUM_PIC_WORD    ; add it to checksum accumulator 
          ; now we write the UDP data
          For DUMB = 0 To (UDP_REPLY_STRING_LEN - 1)  ; loc 43 ... to
              POSTINC2 = UDP_REPLY_STRING[DUMB]       ; 43 + UDP_REPLY_STRING_LEN
          Next
          TEMPG = 43 + UDP_REPLY_STRING_LEN - 1
          CALC_CKSUM_PIC 43, TEMPG
          INTER_CKSUM = INTER_CKSUM + CKSUM_PIC_WORD
          ; the following have to be added as a part of UDP pseudo header (ie 12 bytes), and which have the following content:
          ; - Source IP .. 4 bytes
          ; - Destination IP .. 4 bytes
          ; - blanc filler .. 1 byte
          ; - IP Protocol .. 1 byte (ie UDP $11)
          ; - UDP Length .. 2 bytes (as calculated above)
          POSTINC2 = 0                ; blanc filler
          POSTINC2 = IP_PROT          ; IP Protocol
          POSTINC2 = UDP_LEN.HighByte ; UDP Len highbyte
          POSTINC2 = UDP_LEN.LowByte  ; UDP Len lowbyte
          ;
          ; now we calculate the UDP checksum
          TEMPGG = TEMPG + 4
          Inc TEMPG
          CALC_CKSUM_PIC TEMPG,TEMPGG
          INTER_CKSUM = INTER_CKSUM + CKSUM_PIC_WORD
          CKSUM_PIC_WORD = INTER_CKSUM.Word1 + INTER_CKSUM.Word0
          ;
          FSR2 = VarPtr TX_PRE_BUF
          FSR2 = FSR2 + 41
          POSTINC2 = CKSUM_PIC_WORD.HighByte  ; point at location 41 and write the UDP_CKSUM.Highbyte
          POSTINC2 = CKSUM_PIC_WORD.LowByte   ; and the lowbyte at location 42
          ; we pass now this formatted data from our virtual buffer to ENC28J60 TX buffer
          ; first build the packet to be sent in TX Bufer
          ; following if...endif could be a sub-routine as it is repeating itself thru code !!!!!!!
          If NEW_TX_PACK_PNTR != TX_BUF_START Then  ; if its not the very first Tx packet then
             WRITE_ETH_REGISTER ADR_ETXSTL,NEW_TX_PACK_PNTR.LowByte  ; set TX Buffer START Location 
             WRITE_ETH_REGISTER ADR_ETXSTH,NEW_TX_PACK_PNTR.HighByte ;
             ;
             WRITE_ETH_REGISTER ADR_EWRPTL,NEW_TX_PACK_PNTR.LowByte  ; set the Tx Write pointer
             WRITE_ETH_REGISTER ADR_EWRPTH,NEW_TX_PACK_PNTR.HighByte ;
          EndIf
          ;
          Bcf ETH_CS                                  ; Enable SPI on ENC28J60
          SPI WBM_OP_CODE                             ; Send WBM Opcode on SPI 
          FSR2 = VarPtr TX_PRE_BUF
          DUMB_BYTE = 1 + 42 + UDP_REPLY_STRING_LEN
          For DUMB = 0 To (DUMB_BYTE - 1)
              SPI POSTINC2
          Next
          Bsf ETH_CS
          ; advance pointers
          NEW_TX_PACK_PNTR =  NEW_TX_PACK_PNTR + DUMB_BYTE
          ; add the ETH_TX_TAIL part and start transmission
          GoSub WRITE_ETH_TX_TAIL
          Return
;******************************************************************************
CHECK_PACKET_TYPE Macro
          GoSub CHECK_PACKET_TYPE_SUB
          Endm 
CHECK_PACKET_TYPE_SUB:          
          Select RX_PACKET_TYPE
          Case  ARP_TYPE_HEADER ;$0806  --> ARP type header
                Select ARP_OP ; check ARP Operation (ie Request or Reply)
                Case ARP_OP_REQUEST ;$01   --> its an ARP Request       
                     If ARP_TIPADDR_0 = MYIP_3 Then
                        If ARP_TIPADDR_1 = MYIP_2 Then
                           If ARP_TIPADDR_2 = MYIP_1 Then
                              If ARP_TIPADDR_3 = MYIP_0 Then
                                 ARP_REPLY  ; reply to ARP packet if addressed to this IP
                              EndIf
                           EndIf
                        EndIf
                     EndIf
                Case ARP_OP_REPLY           ;$02  --> its an ARP Reply
                EndSelect
          Case TCP_TYPE_HEADER              ;$0800  --> TCP Type header
                Select IP_PROT
                Case PROT_ICMP              ;$01 --> ICMP
                     Select ICMP_TYPE                               ; check if it is an ICMP request
                     Case ICMP_ECHO_REQUEST ;$08   -->  so it's a ECHO Request
                          If IP_DEST_ADDR_0 = MYIP_3 Then           ; is it My IP ??
                             If IP_DEST_ADDR_1 = MYIP_2 Then        ;
                                If IP_DEST_ADDR_2 = MYIP_1 Then     ;
                                   If IP_DEST_ADDR_3 = MYIP_0 Then  ;
                                      ICMP_REPLY  ; OK, its My IP, so Reply to it
                                   EndIf
                                EndIf
                             EndIf
                          EndIf
                     Case ICMP_ECHO_REPLY   ;$00 --> its a ECHO Reply
                     ;    so do something with that
                     EndSelect
                Case PROT_UDP                                  ; $11 --> UDP
                     If IP_DEST_ADDR_0 = MYIP_3 Then           ; is it My IP ??
                        If IP_DEST_ADDR_1 = MYIP_2 Then        ;
                           If IP_DEST_ADDR_2 = MYIP_1 Then     ;
                              If IP_DEST_ADDR_3 = MYIP_0 Then  ;
                                 ; you can insert here incoming UDP port accepted and test for it
                                 UDP_REPLY     ; OK, its My IP, so Reply to this UDP, or Whatever action
                              EndIf
                           EndIf
                        EndIf
                     EndIf
                EndSelect
          EndSelect
          Return
;******************************************************************************
READ_RX_BUFS Macro
          GoSub READ_RX_BUFS_SUB
          Endm
READ_RX_BUFS_SUB:
          Bcf ETH_CS                                  ; Enable SPI on ENC28J60
          SPI RBM_OP_CODE                             ; Send RBM Opcode on SPI 
          For DUMB = 0 To 5
              RX_PACKET_HEADER[DUMB] = SPI $FF ;READ_BUFFER_MEMORY
          Next
          ;
          RX_BYTE_COUNT = RX_BYTE_COUNT - 4           ; real is (RX_BYTE_COUNT - 4) i.e the last 4 CRC bytes of frame
          ;
          For DUMB = 0 To (RX_BYTE_COUNT - 1)                      
              ETH_COM_PACKET[DUMB] = SPI $FF          ;READ_BUFFER_MEMORY
          Next
          Bsf ETH_CS
          CHECK_PACKET_TYPE  ; Check the received packet header type and act accordingly
          DUMB_WORD = NEXT_PACKET_POINTER  - 1
          WRITE_ETH_REGISTER ADR_ERXRDPTL, DUMB_WORD.LowByte          ; update Rx Buf pointers
          WRITE_ETH_REGISTER ADR_ERXRDPTH, DUMB_WORD.HighByte
          Bsf ECON2_PKTDEC                                            ; decrement the rx packet number
          SET_REGISTER_BIT ADR_ECON2, ECON2
          WRITE_ETH_REGISTER ADR_ERDPTL,NEXT_PACKET_POINTER.LowByte   ; update Nex_Packet_Pointer
          WRITE_ETH_REGISTER ADR_ERDPTH,NEXT_PACKET_POINTER.HighByte
          Return
;******************************************************************************
WRITE_ETH_TX_HEADER:       ; this part is always sent
          ; first build the packet to be sent in TX Bufer
          If NEW_TX_PACK_PNTR != TX_BUF_START Then  ; if its not the very first Tx packet then
             WRITE_ETH_REGISTER ADR_ETXSTL,NEW_TX_PACK_PNTR.LowByte  ; set TX Buffer START Location 
             WRITE_ETH_REGISTER ADR_ETXSTH,NEW_TX_PACK_PNTR.HighByte ;
             ;
             WRITE_ETH_REGISTER ADR_EWRPTL,NEW_TX_PACK_PNTR.LowByte  ; set the Tx Write pointer
             WRITE_ETH_REGISTER ADR_EWRPTH,NEW_TX_PACK_PNTR.HighByte ;
          EndIf
          ;
          Bcf ETH_CS                                  ; Enable SPI on ENC28J60
          SPI WBM_OP_CODE                             ; Send WBM Opcode on SPI
          SPI TX_CONTROL_BYTE   ; write the TX per packet byte at first location.We have the autoinc=ON
          ;common ethernet packet bytes
          SPI SRC_MAC_ADR_0     ; this is the destination MAC, in our case the MAC of ARP requesting machine
          SPI SRC_MAC_ADR_1
          SPI SRC_MAC_ADR_2
          SPI SRC_MAC_ADR_3
          SPI SRC_MAC_ADR_4
          SPI SRC_MAC_ADR_5
          SPI MYMAC_0           ; this is the source MAC, ie this board's MAC 
          SPI MYMAC_1
          SPI MYMAC_2
          SPI MYMAC_3
          SPI MYMAC_4
          SPI MYMAC_5
          SPI RX_PACKET_TYPE.HighByte  ; the packet type , in our case here an ARP ($0806)
          SPI RX_PACKET_TYPE.LowByte
          Bsf ETH_CS
          Return

;******************************************************************************
WRITE_ETH_TX_TAIL:
          WRITE_ETH_REGISTER ADR_ETXNDL, NEW_TX_PACK_PNTR.LowByte    ; set the TX Buffer END pointer to the last byte in packet
          WRITE_ETH_REGISTER ADR_ETXNDH, NEW_TX_PACK_PNTR.HighByte
          Clrf ECON1                ;
          Bsf ECON1_TXRTS           ; start the transmission
          SET_REGISTER_BIT ADR_ECON1, ECON1    ;
          Repeat 
                ECON1 = READ_ETH_REGISTER ADR_ECON1  ; check if transmission finished
          Until ECON1_TXRTS = 0
          NEW_TX_PACK_PNTR = NEW_TX_PACK_PNTR + 7    ;calculate and set the new NEW_TX_PACK_PNTR
          Return
;******************************************************************************
OVER_ETH_MACROS:





